use std::fmt::Debug;

use super::Value;
use crate::interpreter::bytecode::Instruction;

#[derive(Debug, Clone, PartialEq, Eq)]
pub struct Function<'s> {
    pub bytecode: Box<[Instruction]>,
    pub consts: Box<[Value<'s>]>,
    pub locals_len: usize,
}

#[derive(Clone)]
pub struct NativeFun<'s>(pub fn(&[Value<'s>]) -> Value<'s>);
impl<'s> Debug for NativeFun<'s> {
    fn fmt<'a>(&'a self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.debug_tuple("NativeFun")
            .field(&(self.0 as fn(&'a _) -> _))
            .finish()
    }
}

impl<'s> PartialEq for NativeFun<'s> {
    fn eq<'a>(&'a self, other: &Self) -> bool {
        (self.0 as fn(&'a _) -> _) == other.0
    }
}

impl<'s> Eq for NativeFun<'s> {}
