use super::Value;
use std::{fmt::Display, rc::Rc};

#[derive(Debug, Clone, PartialEq, Eq)]
pub struct DotPair<'s>(pub Rc<(Value<'s>, Value<'s>)>);
impl<'s> DotPair<'s> {
    pub fn new(first: Value<'s>, second: Value<'s>) -> Self {
        Self(Rc::new((first, second)))
    }

    pub fn first(&self) -> &Value<'s> {
        &self.0 .0
    }

    pub fn second(&self) -> &Value<'s> {
        &self.0 .1
    }
}

impl<'s> Display for DotPair<'s> {
    fn fmt(mut self: &Self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        "(".fmt(f)?;
        loop {
            self.first().fmt(f)?;
            match self.second() {
                Value::Nil => break,
                Value::DotPair(p) => {
                    " ".fmt(f)?;
                    self = p;
                }
                val => break write!(f, " . {val}")?,
            }
        }
        ")".fmt(f)?;
        Ok(())
    }
}

macro_rules! list {
    () => {
        $crate::interpreter::value::Value::Nil
    };

    ($item: expr $(,)?) => {
        Value::DotPair(
            $crate::interpreter::value::DotPair::new($item, $crate::interpreter::value::Value::Nil)
        )
    };

    ($first: expr, $($rest: expr),* $(,)?) => {
        Value::DotPair(
            $crate::interpreter::value::DotPair::new(
                $first,
                $crate::interpreter::value::pair::list!($($rest),*)
            )
        )
    };
}

pub(crate) use list;
