use crate::{error::Error, interpreter::value::Value, syntax::parser::parse};
use rustyline::{error::ReadlineError, Editor};

pub fn repl() -> rustyline::Result<()> {
    let prompt = "#=> ";
    let mut rl = Editor::<()>::new()?;
    loop {
        match rl.readline(prompt) {
            Ok(line) => match parse(&line) {
                Ok(values) => values
                    .into_iter()
                    .for_each(|e| println!("{}", Into::<Value>::into(e))),
                Err(e) => e.into_iter().map(Error::Parse).for_each(|e| {
                    if let Err(e) = e.report(&line) {
                        eprintln!("Failed to generate error report\n{e:?}")
                    }
                }),
            },
            Err(ReadlineError::Eof) => {
                println!("Bye!");
                break Ok(());
            }
            Err(ReadlineError::Interrupted) => (),
            Err(e) => return Err(e),
        }
    }
}
